#!/bin/sh
# Runs on a regular basis, sets RTC for scheduled wakeup (if the musicserver is shut down on evening) and notifies AUTO_UPDATER.sh via /home/updates/pending

# if Autoupdate is not configured remove the RTC and pending file and exit
AUTOUPD=`cat /etc/loxone.conf | grep AUTOUPDATE= | cut -d= -f2`
if [ "${AUTOUPD}" -eq "0" ] ; then
    echo "Autoupdates disabled"
    echo 0 > /sys/class/rtc/rtc0/wakealarm
    rm -rf /home/updates/pending
    exit 0
fi

# If previous iteration did everything right, there is nothing more to do
CURRALARM=`cat /sys/class/rtc/rtc0/wakealarm`
if [ "${CURRALARM}" != "" ] && [ -f /home/updates/pending ] ; then
    echo "There is already an alarm configured, nothing to do"
    exit 0 
fi

# Check for update availability
MODEL=`cat /etc/loxone.conf | grep MODEL= | cut -d= -f2`
VERSION=`cat /etc/loxone.conf | grep VERSION_LWSS= | cut -d= -f2`
UPDATELEVEL=`cat /etc/loxone.conf | grep UPDATELEVEL= | cut -d= -f2`
VERSION=`echo ${VERSION} | awk 'BEGIN {FS="."} {printf("%d%02d%02d%02d",$1,$2,$3,$4)}'`
ISUPDATEAVAIL=`wget --tries=3 --timeout=5 -qO- "http://mediaupdate.loxone.com/preflight/?model=${MODEL}&version=${VERSION}&updatelevel=${UPDATELEVEL}&bool=1"`

if [ "${ISUPDATEAVAIL}" -eq "1" ] ; then
    # calculate next rtc alarm
    CURR_YMD=`date '+%Y-%m-%d'`
    CURR_TS=`date '+%s' -d "${CURR_YMD} 00:00:00"`
    # Set Timer to: 5 Minutes before scheduled update time 
    NEXT_AUTOUPD=$(($CURR_TS+86400+($AUTOUPD*60) - 300))
    echo 0 > /sys/class/rtc/rtc0/wakealarm
    echo ${NEXT_AUTOUPD} > /sys/class/rtc/rtc0/wakealarm
    SCHEDULE=`date -d @${NEXT_AUTOUPD}`
    touch /home/updates/pending
    echo "Scheduling RTC WakeUp for ${SCHEDULE}"
else
    echo "No Update Available"
fi

