$(document).ready(function () {
    var server = window.location;
    var statePanel = {0: 'info', 1: 'success', 2: 'warning', 3: 'danger'};
    var stateGlyphicons = {0: 'glyphicon-question', 1: 'glyphicon-ok', 2: 'glyphicon-alert', 3: 'glyphicon-remove'};
    var config = {};

    var CallBacks = {
        System: $.Callbacks(),
        Network: $.Callbacks(),
        Application: $.Callbacks(),
        Services: $.Callbacks()
    };
    var WsCallBack = $.Callbacks();

    var UpdateCallBack = $.Callbacks();
    var Intervals = {
        UpdCheck: 0,
        Reboot: 0,
        Restart: 0,
        WsConnRetry: 0,
        WsMonitoringConnRetry: 0,
        StatesReload: 0
    };
    var States = {
        Initial: true,
        Websocket: false,
        MonitoringWebsocket: false,
        Monitoring: {}
    };
    var Health = {
        system: {
            workload: true,
            storage: true,
            logitech: true,
            lms: true,
            all: 1
        },
        network: {
            local: true,
            miniserver: true,
            internet: true,
            all: 1
        },
        zones: {
            all: 1
        }
    };
    CallBacks.Application.add(ApplicationStateDetails);
    CallBacks.System.add(SystemStateDetails);
    CallBacks.Network.add(NetworkStateDetails);
    CallBacks.Services.add(ServiceStateDetails);

    WsCallBack.add(ZoneStateDetails);
    WsCallBack.add(PlayerState);
    UpdateCallBack.add(showUpdateDialog);

    if ($("div#panel-status").attr("data-disabled") !== undefined) {
        $("div#zoneState").css('display', 'none');
    } else
    {
        var ws = StartWebSocket();
    }

    /* Load Language File */
    $.ajax({
        url: window.location.href,
        data: {action: 'getConfig'},
        dataType: 'json'
    }).done(function (result) {
        config = result;
        config.lang = (result.lang) ? result.lang : "ENG";
        $.ajax({
            url: "/lang/" + config.lang + ".json",
            dataType: 'json'
        }).done(function (result) {
            config.langStrings = result;
            SetAppStrings();
            IsUpdateAvailable(true);
            IsUpdateLevelChanged();
            IsForceUpdateRunning();
            StartMonitoringWebSocket();
        }).fail(function () {
            $.ajax({
                url: "/lang/ENG.json",
                dataType: 'json'
            }).done(function (result) {
                config.langStrings = result;
                SetAppStrings();
                IsUpdateAvailable(true);
                IsUpdateLevelChanged();
                IsForceUpdateRunning();
                StartMonitoringWebSocket();
            });
        });
    });

    /* ACTION TAB - Reboot */
    $("#btn-reboot").on('click', function () {
        restartServer(true, server.hostname);
    });

    /* ACTION TAB - Restart */
    $("#btn-restart").on('click', function () {
        $("#state-restart").html(_L('modal.restart.state.inprogress')).css('color', 'orange');
        $("#btn-restart").prop('disabled', true);
        $.ajax({
            url: server.location,
            data: {action: 'restartservices'},
            dataType: "json"
        });
        Intervals.Restart = setInterval(function () {
            var ws = getWS(server.hostname, 7091);
            try {
                ws.onopen = function () {
                    $("#state-restart").html(_L('modal.restart.state.completed')).css('color', 'green');
                    clearInterval(Intervals.Restart);
                    ws.onclose = function () {};
                    ws.close();
                    ws = StartWebSocket();
                };
            } catch (err) {

            }
        }, 5000);
    });
    /* ACTION TAB - Open Update Dialog */
    $("button#btn-update").on('click', function () {
        $("button#btn-update").prop('disabled', true);
        $.ajax({
            url: window.location.href,
            data: {action: 'doUpdate'},
            dataType: 'json'
        }).done(function (updatestatus) {
            $("#state-update").html(_L('modal.update.state.inprogress.1')).css('color', 'orange');
            Intervals.UpdCheck = setInterval(function () {
                $.ajax({
                    url: window.location.href,
                    data: {action: 'getUpdateStatus'},
                    dataType: 'json'
                }).done(function (response)
                {
                    processUpdateResponse(response);
                });
            }, 10000);
        });
    });
    /* SETTINGS TAB */
    $("div#networksettings > div.panel-heading").on('click', function () {
        if ($("div#networksettings > div.panel-body").attr('data-state') === "collapsed")
        {
            $("div#networksettings > div > div.panel-title > span").removeClass('glyphicon-triangle-bottom').addClass('glyphicon-triangle-top');
            $("div#networksettings > div.panel-body").html($("#networkSettingsDetails").html());
            $("div#networksettings > div.panel-body").attr('data-state', 'expanded');
        } else {
            $("div#networksettings > div > div.panel-title > span").removeClass('glyphicon-triangle-top').addClass('glyphicon-triangle-bottom');
            $("div#networksettings > div.panel-body").html(_L('panel.settings.networkconfig.description.collapsed')).attr('data-state', 'collapsed');
        }
    });
    /* Handling for AutoUpdate Settings */
    $("input#autoupdate-schedule").on('change', function () {
        $.ajax({
            url: window.location.href,
            data: {action: 'setAutoUpdate', autoupdate: $(this).prop('checked')},
            dataType: 'json'
        }).done(function (state) {
            if (state.autoupdate === "true") {
                $("input#autoupdate-schedule").next("h5").html(_L('panel.settings.autoupdate.schedule', ["02:00"]));
            } else
                $("input#autoupdate-schedule").next("h5").html(_L('panel.settings.autoupdate.schedule.disabled'));
        });
    });
    // click function - save settings
    $("div#networksettings").on('click', "#btn-save-settings", function () {
        $("div.validationenabled").removeClass('has-error');
        $("div.validationenabled > span").removeClass('glyphicon-remove');
        // get new settings
        var newname = $("input#name").val();
        var newip = $("input#ip").val();
        var newsubnet = $("input#subnet").val();
        var newgateway = $("input#gateway").val();
        var newdns = $("input#dns").val();
        // validate inputs
        var name = validateName($("input#name").val());
        var ipaddress = validateIP($("input#ip").val());
        var subnet = validateIP($("input#subnet").val());
        var gateway = validateIP($("input#gateway").val());
        var dns = validateDNS($("input#dns").val());
        // check for errors
        if ((!ipaddress) && (!$("input#ip").attr('disabled'))) {
            $("div#ip").addClass('has-error');
            $("div#ip > span").addClass('glyphicon-remove');
        }
        if ((!subnet) && (!$("input#subnet").prop('disabled'))) {
            $("div#subnet").addClass('has-error')
            $("div#subnet > span").addClass('glyphicon-remove');
        }
        if ((!gateway) && (!$("input#gateway").prop('disabled'))) {
            $("div#gateway").addClass('has-error');
            $("div#gateway > span").addClass('glyphicon-remove');
        }
        if ((!dns) && (!$("input#dns").prop('disabled'))) {
            $("div#dns").addClass('has-error');
            $("div#dns > span").addClass('glyphicon-remove');
        }
        if ((!name) && (!$("input#name").prop('disabled'))) {
            $("div#name").addClass('has-error');
            $("div#name > span").addClass('glyphicon-remove');
        }

// If no errors

        if (!$("div").hasClass('has-error')) {
            var mode = $("input[name='mode']:checked").attr('id');
            if (mode === "mode_dhcp")
                var data = {
                    action: 'resetdhcp',
                    hostname: $("input#name").val()
                }
            else
                var data = {
                    action: 'savesettings',
                    hostname: $("input#name").val(),
                    ip: $("input#ip").val(),
                    subnet: $("input#subnet").val(),
                    gateway: $("input#gateway").val(),
                    dns: $("input#dns").val()

                }
            $.ajax({
                url: server.location,
                data: data,
                dataType: "json"
            });
            $("#modalServerReboot").modal().show();
            $("p#description-reboot").html(_L('modal.reboot.description.fromNWChange'));

            if (mode === "mode_static")
                restartServer(false, $("input#ip").val());
            else
                restartServer(false);

        }

    });
    /* EVENT HANDLERS */

    $("button#btn-action-update").on('click', function () {
        IsUpdateAvailable(false);
    });

    $("button#btn-action-restart").on('click', function () {
        $("div#modalServiceRestart").modal('show');
    });

    $("button#btn-action-reboot").on('click', function () {
        $("div#modalServerReboot").modal('show');
    });

    $("button#btn-action-docs").on('click', function () {
        window.open("http://www.loxone.com/help/music-server");
    });

    /* Reset Update Modal when closed */
    $("#modalUpdate").on('hidden.bs.modal', function () {
        clearInterval(Intervals.UpdCheck);
    });
    /* Reset Reboot Modal when closed */
    $("#modalServerReboot").on('hidden.bs.modal', function () {
        clearInterval(Intervals.Reboot);
    });
    /* Reset Restart Modal when closed */
    $("#modalServiceRestart").on('hidden.bs.modal', function () {
        clearInterval(Intervals.Restart);
    });

    $("input#name").on('blur', function () {
        $("span#help_name").hide();
    });

    $("div#networksettings").on('click', 'input#mode_dhcp', function () {

        $("input#ip").prop('disabled', true);
        $("input#subnet").prop('disabled', true);
        $("input#gateway").prop('disabled', true);
        $("input#dns").prop('disabled', true);
    });
    $("div#networksettings").on('click', "input#mode_static", function () {

        $("input#ip").prop('disabled', false);
        $("input#subnet").prop('disabled', false);
        $("input#gateway").prop('disabled', false);
        $("input#dns").prop('disabled', false);
    });
    /* FUNCTIONS */

    function IsUpdateLevelChanged() {
        if ($("div#modalUpdLvlChg").attr('data-changedto') !== "0")
            $("div#modalUpdLvlChg").modal('show');
    }

    function IsForceUpdateRunning() {
        if ($("div#modalUpdate").attr('data-forceupdate') !== "0") {
            $("div#modalUpdate").modal('show').on("shown.bs.modal", function () {
                $("div#modalUpdate").find("#btn-update").prop('disabled', true);
                $("div#modalUpdate").find("#state-update").html(_L('modal.update.state.inprogress.1')).css('color', 'orange');
                $("div#modalUpdate").find("h4.modal-title").html(_L('modal.update.title'));
                $("div#modalUpdate").find("p#description-update").html(_L('modal.update.description.4'));
                Intervals.UpdCheck = setInterval(function () {
                    $.ajax({
                        url: window.location.href,
                        data: {action: 'getUpdateStatus'},
                        dataType: 'json'
                    }).done(function (response)
                    {
                        processUpdateResponse(response);
                    });
                }, 10000);
            });
        }
    }

    function showUpdateDialog(update) {
        $("div#modalUpdate").modal('show');
        if (update.version !== "0") {
            $("div#modalUpdate").find("p#description-update").html(_L('modal.update.description.2', [update.desc, byteSizeFormat(parseInt(update.size / 1024)), update.eta]));
            $("div#modalUpdate").find("button#btn-update").prop('disabled', false);
        }
    }

    function SetAppStrings() {
        /* Prepare Navigation Strings */
        $("li#tab-status > a").html(_L('nav.status'));
        $("li#tab-settings > a").html(_L('nav.settings'));
        $("li#tab-actions > a").html(_L('nav.actions'));

        $("button#btn-action-update > span.pull-left").html(_L('nav.actions.button.update'));
        $("button#btn-action-restart > span.pull-left").html(_L('nav.actions.button.restart'));
        $("button#btn-action-reboot > span.pull-left").html(_L('nav.actions.button.reboot'));
        $("button#btn-action-docs > span.pull-left").html(_L('nav.actions.button.docs'));

        /* Panels */
        $("div#autoupdates").find("div.panel-title").html(_L('panel.settings.autoupdate.title'));
        $("p#updatelevel-revert").html(_L('panel.settings.autoupdate.reverttorelease', [(config.updatelevel).toUpperCase()]));
        $("div#networksettings").find("div.panel-title").html(_L('panel.settings.networkconfig.title'));
        $("div#networksettings").find("div.panel-body").html(_L('panel.settings.networkconfig.description.collapsed'));
        $("div#networkSettingsDetails > h5").html(_L('panel.settings.networkconfig.ipassignmentmode'));
        $("input#mode_static").after(_L('panel.settings.networkconfig.ipassignmentmode.static'));
        $("input#mode_dhcp").after(_L('panel.settings.networkconfig.ipassignmentmode.dynamic'));
        $("div#networkSettingsDetails").find("div#name > h5").html(_L('panel.settings.networkconfig.hostname'));
        $("div#networkSettingsDetails").find("div#ip > h5").html(_L('panel.settings.networkconfig.address'));
        $("div#networkSettingsDetails").find("div#subnet > h5").html(_L('panel.settings.networkconfig.subnet'));
        $("div#networkSettingsDetails").find("div#gateway > h5").html(_L('panel.settings.networkconfig.router'));
        $("div#networkSettingsDetails").find("div#dns > h5").html(_L('panel.settings.networkconfig.dns'));
        $("button#btn-save-settings").html(_L('panel.settings.networkconfig.savesettings'));
        $("div#systemState").find("div.panel-title").html(_L('panel.status.system.title') + '<span class="glyphicon pull-right" />');
        $("div#systemState").find("tr#health-uptime > td:first").html(_L('panel.status.system.uptime'));
        $("div#systemState").find("tr#health-cpu > td:first").html(_L('panel.status.system.cpu'));
        $("div#systemState").find("tr#health-disk > td:first").html(_L('panel.status.system.disk'));
        $("div#systemState").find("tr#health-logitech > td:first").html(_L('panel.status.system.logitech'));
        $("div#systemState").find("tr#health-lms > td:first").html(_L('panel.status.system.lms'));
        $("div#systemState").find("tr#health-masterms > td:first").html(_L('panel.status.system.masterms'));
        $("div#networkState").find("div.panel-title").html(_L('panel.status.network.title') + '<span class="glyphicon pull-right" />');
        $("div#networkState").find("tr#health-nw-gw > td:first").html(_L('panel.status.network.gw'));
        $("div#networkState").find("tr#health-nw-ms > td:first").html(_L('panel.status.network.ms'));
        $("div#networkState").find("tr#health-nw-inet > td:first").html(_L('panel.status.network.inet'));
        $("div#zoneState").find("div.panel-title").html(_L('panel.status.zones.title') + '<span class="glyphicon pull-right" id="glyphicon-zoneState" />');

        /* Prepare Modal Dialog Strings */
        $("div#modalServiceRestart").on('show.bs.modal', function () {
            $(this).find("#btn-restart").prop('disabled', false);
            $(this).find("h4.modal-title").html(_L('modal.restart.title'));
            $(this).find("p#description-restart").html(_L('modal.restart.description'));
            $(this).find("p#state-restart").html(_L('modal.restart.state.idle')).css('color', '');
            $(this).find("button#btn-cancel").html(_L('modal.button.close'));
            $(this).find("button#btn-restart").html(_L('modal.button.restart'));
        });

        $("div#modalServerReboot").on('show.bs.modal', function () {
            $(this).find("#btn-reboot").prop('disabled', false);
            $(this).find("h4.modal-title").html(_L('modal.reboot.title'));
            $(this).find("p#description-reboot").html(_L('modal.reboot.description'));
            $(this).find("p#state-reboot").html(_L('modal.reboot.state.idle')).css('color', '');
            $(this).find("button#btn-cancel").html(_L('modal.button.close'));
            $(this).find("button#btn-reboot").html(_L('modal.button.reboot'));
        });

        $("div#modalUpdate").on('show.bs.modal', function () {
            $(this).find("#btn-update").prop('disabled', true);
            $(this).find("#state-update").html('').css('color', '');
            $(this).find("h4.modal-title").html(_L('modal.update.title'));
            $(this).find("p#description-update").html(_L('modal.update.description.1'));
            $(this).find("button#btn-cancel").html(_L('modal.button.close'));
            $(this).find("button#btn-update").html(_L('modal.button.update'));
            $(this).find("a#changelog").html((_L('modal.update.changelog')).toUpperCase());
        });

        $("div#modalUpdLvlChg").on('show.bs.modal', function () {
            $(this).find("h4.modal-title").html(_L('modal.updlvlchg.title'));
            $(this).find("div.modal-body > p").html(_L('modal.updlvlchg.description.1', [($(this).attr('data-changedto')).toUpperCase()]));
            $(this).find("button#btn-cancel").html(_L('modal.button.close'));
        });
    }

    function IsUpdateAvailable(bStart) {
        var arrVersion = (config.version_lwss).split(".");
        var version = arrVersion[0] + ((arrVersion[1].length == 1) ? "0" + arrVersion[1] : arrVersion[1]) + ((arrVersion[2].length == 1) ? "0" + arrVersion[2] : arrVersion[2]) + ((arrVersion[3].length == 1) ? "0" + arrVersion[3] : arrVersion[3]);
        var updatelevel = config.updatelevel;
        var updatetest = config.updatetest;
        if (parseInt(config.autoupdate) > 0) {
            $("input#autoupdate-schedule").prop('checked', true);
            $("input#autoupdate-schedule").next("h5").html(_L('panel.settings.autoupdate.schedule', ["02:00"]));
        } else
            $("input#autoupdate-schedule").next("h5").html(_L('panel.settings.autoupdate.schedule.disabled'));
        $.ajax({
            url: "http://mediaupdate.loxone.com/preflight/index.php",
            data: {version: version, model: config.model, updatelevel: updatelevel, updatetest: updatetest},
            dataType: 'jsonp'
        }).done(function (update) {
            if ((update.version !== "0") || (!bStart))
                UpdateCallBack.fire(update);
        });

    }

    function processUpdateResponse(response) {

        if (response.flag == "CONT") {
            /* currently no more info */
        } else if (response.flag == "FAIL") {
            clearInterval(Intervals.UpdCheck);
            $("#state-update").html(_L('modal.update.state.error.1')).css('color', 'red');
        } else if (response.flag == "FIN") {
            clearInterval(Intervals.UpdCheck);
            if (!States.Websocket) {
                $("#state-update").html(_L('update.state.inprogress.2')).css('color', 'orange');
                Intervals.Restart = setInterval(function () {
                    if (States.Websocket) {
                        clearInterval(Intervals.Restart);
                        $("#description-update").html(_L('modal.update.description.3'));
                        $("#state-update").html('<b>' + _L('modal.update.state.completed') + '</b>').css('color', 'green');
                    }
                }, 2500);
            }
        }
    }

    function restartServer(sendCommand, redirectIP) {
        $("#state-reboot").html(_L('modal.reboot.state.inprogress')).css('color', 'orange');
        $("#btn-reboot").prop('disabled', true);
        if (sendCommand === true)
            $.ajax({
                url: window.location.location,
                data: {action: 'restartserver'},
                dataType: "json"
            });
        // If dhcp is used we do not know the ip, therefore we show a message after 60s 
        if (redirectIP !== undefined) {
            if (redirectIP !== window.location.host)
                var bIPChanged = true;
            else
                var bIPChanged = false;
            Intervals.Reboot = setInterval(function () {
                var ws = getWS(redirectIP, 7091, "remotecontrol");
                try {
                    ws.onopen = function () {
                        clearInterval(Intervals.Reboot);
                        ws.onclose = function () {};
                        ws.close();
                        // If the IP has changed (static ip change) we would redirect the user once we see the websocket being online again
                        if (bIPChanged) {
                            $("#state-reboot").html(_L('modal.reboot.state.completed', [redirectIP])).css('color', 'green');
                            setTimeout(function () {
                                window.location.replace("http://" + redirectIP);
                            }, 5000);
                        } else
                            $("#state-reboot").html(_L('modal.reboot.state.completed.2')).css('color', 'green');
                    };
                } catch (err) {

                }
            }, 5000);
        } else {
            setTimeout(function () {
                $("#state-reboot").html(_L('modal.reboot.state.completed.3')).css('color', 'green');
            }, 60000);
        }
    }
    /*
     * AUDIO ZONES SECTION
     */

    function ZoneStateDetails() {
        if (!(States.configall_result && States.getplayers_result && States.Initial === true))
            return;
        States.Initial = false;
        setLWSState(1);
        var config = States.configall_result[0];
        var players = States.getplayers_result;
        var Player = {};
        var table;
        $.each(config.players, function (idx, zone) {
            Player[zone.playerid] = {};
            Player[zone.playerid].enabled = zone.enabled;
            Player[zone.playerid].name = zone.name;
            Player[zone.playerid].type = zone.clienttype;
        })
        $.each(players, function (idx, player) {
            Player[player.playerid].mode = player.mode;
            Player[player.playerid].power = player.power;
        })
        $('div#zoneStateDetail > table').empty();
        $.each(Player, function (zoneid, player) {
            var name = player.name;
            if (player.type === 1) // UPnP Zone Type
                name = "<span class=\"fa fa-wifi\" style=\"padding-right:2px;\"></span>" + name;
            $("table#zones").append("<tr><td>" + name + "</td><td align='right' id='zone" + zoneid + "'></td></tr>");
            UpdatePlayerStatus(zoneid, player);
        })
    }

    function UpdatePlayerStatus(zoneid, player) {

        if (!player.enabled) {
            var state = _L('panel.status.zones.zone.disabled');
            var color = "#aaa";
        } else {
            if (player.mode == "play") {
                var state = _L('panel.status.zones.zone.play');
                var color = "#444";
            } else {

                if (player.power == "offline") {
                    var state = _L('panel.status.zones.zone.offline');
                    var color = "#aaa";
                } else {
                    var state = _L('panel.status.zones.zone.stop');
                    var color = "#444";
                }
            }
        }
        $("td#zone" + zoneid).html(state).css('color', color);
    }

    // Update Player State
    function PlayerState(command) {
        if (command === "audio_event") {
            var event = States.audio_event[0];
            UpdatePlayerStatus(event.playerid, {mode: event.mode, power: event.power, enabled: 1});
        }
    }

    /*
     * SYSTEM STATE SECTION
     */

    function setSystemStateAlarm() {
        var level = Math.max(States.Monitoring.cpu, States.Monitoring.disk, States.Monitoring.LWS, States.Monitoring.LMS);
        if (isNaN(level))
            level = 1;
        $("div#systemState").attr('class', 'panel').addClass('panel-' + statePanel[level]);
        $("div#systemState > div.panel-heading > div.panel-title > span").attr('class', 'pull-right glyphicon').addClass(stateGlyphicons[level]);
    }

    function setLWSState(state) {
        States.Monitoring.LWS = state;
        if (state === 1)
            $("#systemStateDetail-LMS").html(_L('panel.status.system.lms.state.1')).css('color', '');
        else
            $("#systemStateDetail-LMS").html(_L('panel.status.system.lms.state.3')).css('color', 'red');

    }

    function setLMSState(state) {
        if (state === 1)
            $("#systemStateDetail-Logitech").html(_L('panel.status.system.logitech.state.1')).css('color', '');
        else
            $("#systemStateDetail-Logitech").html(_L('panel.status.system.logitech.state.3')).css('color', 'red');
    }

    function setCPUState(state) {
        if (state == 1)
            $("#systemStateDetail-CPULoad").html(_L('panel.status.system.cpu.state.1')).css('color', '');
        if (state == 2)
            $("#systemStateDetail-CPULoad").html(_L('panel.status.system.cpu.state.2')).css('color', 'orange');
        if (state == 3)
            $("#systemStateDetail-CPULoad").html(_L('panel.status.system.cpu.state.3')).css('color', 'red');
        return state;
    }

    function setDISKState(state) {
        if (state <= 2)
            $("#systemStateDetail-DISKLoad").html('~ ' + parseInt(parseInt((States.Monitoring.system.disk.Free / 0.0041) / 1000) * 1000).formatNumber(0, '', '.') + ' mp3 ' + _L('panel.status.system.songcaption') + '<br/>(' + States.Monitoring.system.disk.Free + ' GB)').css('color', '');
        if (state == 3)
            $("#systemStateDetail-DISKLoad").html(_L('panel.status.system.disk.state.3')).css('color', 'red');
        return state;
    }



    function SystemStateDetails()
    {
        /* UPTIME */
        $("#systemStateDetail-Uptime").html(formatUptime(States.Monitoring.system.cpu.Uptime));

        /* CPU */
        var cpuload = parseFloat(States.Monitoring.system.cpu.Avg5 * 100) / States.Monitoring.system.cpu.Cores;
        if (cpuload >= 95)
            States.Monitoring.cpu = setCPUState(3);
        else if ((cpuload < 95) && (cpuload > 75))
            States.Monitoring.cpu = setCPUState(2);
        else
            States.Monitoring.cpu = setCPUState(1);

        /* DISK */
        if ((States.Monitoring.system.disk.Free < 5) && (States.Monitoring.system.disk.Free > 2))
            States.Monitoring.disk = setDISKState(2);
        else if (States.Monitoring.system.disk.Free <= 2)
            States.Monitoring.disk = setDISKState(3);
        else
            States.Monitoring.disk = setDISKState(1);

        /* LOGITECH */
        if (States.Monitoring.system.serviceLMS === true)
            States.Monitoring.LMS = setLMSState(1);
        else
            States.Monitoring.LMS = setLMSState(3);
        setSystemStateAlarm();
    }



    /*
     * APP STATE SECTION 
     */

    function setApplicationStateAlarm(level) {
        $("#zoneState").attr('class', 'panel').addClass('panel-' + statePanel[level]);
        $("#glyphicon-zoneState").attr('class', 'glyphicon pull-right').addClass(stateGlyphicons[level]);
    }

    function ApplicationStateDetails() {

    }

    /*
     * NETWORK STATE SECTION
     */
    function setNetworkStateAlarm() {
        var level = Math.max(States.Monitoring.internet, States.Monitoring.gateway, States.Monitoring.miniserver);
        $("div#networkState").attr('class', 'panel').addClass('panel-' + statePanel[level]);
        $("div#networkState > div.panel-heading > div.panel-title > span").attr('class', 'pull-right glyphicon').addClass(stateGlyphicons[level]);
    }

    function setPingGateway(state, results) {
        $("#networkStateDetail-LPing").attr('title', "Loss: " + results.Loss + "%, Time: " + results.Min + "ms");
        $("#networkStateDetail-LPing").html(_L('panel.status.network.gw.state.1')).css('color', '');
        if (state == 3)
            $("#networkStateDetail-LPing").html(_L('panel.status.network.gw.state.3')).css('color', 'red');
        return state;
    }

    function setPingInternet(state, results) {
        $("#networkStateDetail-RPing").attr('title', "Loss: " + results.Loss + "%, Time: " + results.Min + "ms");
        $("#networkStateDetail-RPing").html(_L('panel.status.network.inet.state.1')).css('color', '');
        if (state == 3)
            $("#networkStateDetail-RPing").html(_L('panel.status.network.inet.state.3')).css('color', 'red');
        return state;
    }

    function setPingMiniserver(state, results) {
        $("#networkStateDetail-MSPing").attr('title', "Loss: " + results.Loss + "%, Time: " + results.Min + "ms");
        $("#networkStateDetail-MSPing").html(_L('panel.status.network.ms.state.1')).css('color', '');
        if (state == 3)
            $("#networkStateDetail-MSPing").html(_L('panel.status.network.ms.state.3')).css('color', 'red');
        return state;
    }

    function NetworkStateDetails() {

        /* INTERNET */
        if ((States.Monitoring.network.internet === -1) || (States.Monitoring.network.internet.Loss > 50) || (States.Monitoring.network.internet.Min > 100))
            States.Monitoring.internet = setPingInternet(3, States.Monitoring.network.internet);
        else
            States.Monitoring.internet = setPingInternet(1, States.Monitoring.network.internet);

        /* GATEWAY */
        if ((States.Monitoring.network.gateway === -1) || (States.Monitoring.network.gateway.Loss > 0) || (States.Monitoring.network.gateway.Min > 25))
            States.Monitoring.gateway = setPingGateway(3, States.Monitoring.network.gateway);
        else
            States.Monitoring.gateway = setPingGateway(1, States.Monitoring.network.gateway);

        /* MINISERVER */
        if ((States.Monitoring.network.miniserver === -1) || (States.Monitoring.network.miniserver.Loss > 25) || (States.Monitoring.network.miniserver.Min > 50))
            States.Monitoring.miniserver = setPingMiniserver(3, States.Monitoring.network.miniserver);
        else
            States.Monitoring.miniserver = setPingMiniserver(1, States.Monitoring.network.miniserver);

        setNetworkStateAlarm();
    }

    function ServiceStateDetails()
    {
        var version =  States.Monitoring.services.version;
        version = version.replace(/\.0/g,"\.");
        $("div#versiontag > p").html("Version: " + version);
        if (States.Monitoring.services.updatelevel != "release")
            $("div#versiontag > p").html("Version: " + version + " (" + (States.Monitoring.services.updatelevel).toUpperCase() + ")" );
    }

    /*
     * VARIOUS HELPERS
     */

    // validate ip input
    function validateIP(ip) {
        var str = ip;
        var pattern = /^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]).){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$/g;
        var validip = pattern.test(str);
        return validip;
    }

    // validate name input
    function validateName(name) {
        var string = name;
        var pattern = /^[0-9a-zA-z]+$/;
        var validName = pattern.test(string);
        return validName;
    }

    function validateDNS(dns) {
        var pattern = /^[0-9a-zA-Z\.\-_]+$/;
        return pattern.test(dns);
    }

    function byteSizeFormat(bytes) {
        return bytes.toString().replace(/\B(?=(\d{3})+(?!\d))/g, ".");
    }

    function getWS(server, port, protocol)
    {
        var socket = "";
        var readyState = false;
        var url = 'ws://' + server + ':' + port;
        if (typeof MozWebSocket != "undefined") {
            socket = new MozWebSocket(url, protocol);
        } else {
            socket = new WebSocket(url, protocol);
        }
        return socket;
    }

    function StartMonitoringWebSocket() {
        var ws = getWS(server.hostname, 7081, "monitoring");
        try {
            ws.onopen = function () {
                ws.send('statscache');
                ws.onmessage = function (event) {
                    var data = jQuery.parseJSON(event.data);
                    var command = null;
                    console.log(data);
                    $.each(data, function (dst, states) {
                        States.Monitoring[dst] = states;
                        if (dst.match(/system/))
                            CallBacks.System.fire();
                        if (dst.match(/network/))
                            CallBacks.Network.fire();
                        if (dst.match(/services/))
                            CallBacks.Services.fire();
                    });
                    States.MonitoringWebsocket = true;
                };
            };
            ws.onerror = function () {
                MonitoringSocketError();
            };
            ws.onclose = function () {
                MonitoringSocketError();
            }
        } catch (err) {
            MonitoringSocketError();
        }
        return ws;
    }


    function MonitoringSocketError() {
        clearTimeout(Intervals.WsMonitoringConnRetry);
        States.MonitoringWebsocket = false;
        Intervals.WsMonitoringConnRetry = setTimeout(function () {
            States.Initial = true;
            wsm = StartMonitoringWebSocket();
        }, 5000);
    }

    function StartWebSocket() {
        var ws = getWS(server.hostname, 7091, "remotecontrol");
        try {
            ws.onopen = function () {
                ws.send('audio/cfg/all');
                ws.send('audio/cfg/getplayers');
                ws.onmessage = function (event) {
                    try {
                        var data = jQuery.parseJSON(event.data);
                        var command = null;
                        $.each(data, function (idx, value) {
                            if ($.isArray(value)) {
                                States[idx] = value;
                                States.lastCommand = idx;
                                command = idx;
                            }
                        });
                        States.Websocket = true;
                        $("#socketerror").remove();
                        setLWSState(1);
                        setApplicationStateAlarm(1);
                        setSystemStateAlarm();
                        WsCallBack.fire(command);
                    } catch (e)
                    {
                        //console.log("non-json answer");
                    }
                };
            };
            ws.onerror = function () {
                showError();
            };
            ws.onclose = function () {
                showError();
            }
        } catch (err) {
            showError();
        }
        return ws;
    }

    function showError() {
        clearTimeout(Intervals.WsConnRetry);
        States.Websocket = false;
        if (!$("#socketerror").length) {
            $('div#zoneStateDetail > table').empty().after('<p class="text-center" id="socketerror">' + _L('panel.status.zones.error.1') + '</p>');
            setLWSState(3);
            setApplicationStateAlarm(3);
            setSystemStateAlarm();
        }
        Intervals.WsConnRetry = setTimeout(function () {
            States.Initial = true;
            ws = StartWebSocket();
        }, 5000);
    }

    /* shortcut for config.langStrings['id'] */

    function _L(id, vars) {
        if (typeof (vars) === "object") {
            var ufString = config.langStrings[id];
            var i = 0;
            var fString = ufString.replace(/(\%[ds]+)/gi, function (x) {
                return vars[i++];
            });
            return fString;
        } else
            return config.langStrings[id];

    }

    function formatUptime(uptime) {
        var days = parseInt(uptime / 86400);
        var seconds = parseInt(uptime % 86400);
        var hours = parseInt(seconds / 3600);
        seconds = parseInt(seconds % 3600);
        var minutes = parseInt(seconds / 60);
        var fuptime = {
            days: days,
            hours: hours,
            minutes: minutes
        };
        var formattedUptime = "";
        if (days > 0)
        {
            formattedUptime += days + "d ";
            //formattedUptime += (days > 1) ? _L('common.datetime.days') : _L('common.datetime.day');
        }
        /*
         if (hours > 0)
         {
         if (formattedUptime.length > 0)
         formattedUptime += ", ";
         formattedUptime += hours + " ";
         formattedUptime += (hours > 1) ? _L('common.datetime.hours') : _L('common.datetime.hour');
         }
         if (minutes > 0)
         {
         if (formattedUptime.length > 0)
         formattedUptime += " ";
         formattedUptime += minutes + " ";
         formattedUptime += (minutes > 1) ? _L('common.datetime.minutes') : _L('common.datetime.minute');
         }*/
        formattedUptime += " " + hours + "h " + minutes + "m";

        return formattedUptime;
    }
});


Number.prototype.formatNumber = function (c, d, t) {
    var n = this,
            c = isNaN(c = Math.abs(c)) ? 2 : c,
            d = d == undefined ? "." : d,
            t = t == undefined ? "," : t,
            s = n < 0 ? "-" : "",
            i = parseInt(n = Math.abs(+n || 0).toFixed(c)) + "",
            j = (j = i.length) > 3 ? j % 3 : 0;
    return s + (j ? i.substr(0, j) + t : "") + i.substr(j).replace(/(\d{3})(?=\d)/g, "$1" + t) + (c ? d + Math.abs(n - i).toFixed(c).slice(2) : "");
};